/*============================================================================
  File:     BufferPool.sql

  Summary:  Sys.dm_os_buffer_descriptors

  Date:     March 2011

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Load some data
SELECT COUNT (*) from SalesDB..Sales;

-- Basic DMV
SELECT * FROM sys.dm_os_buffer_descriptors;
GO

-- Explain about DBID 32767

-- Now to see aggregated by database
SELECT *,
	DirtyPageCount * 8 / 1024 AS DirtyPageMB,
	CleanPageCount * 8 / 1024 AS CleanPageMB
FROM
	(SELECT 
		(CASE WHEN ([database_id] = 32767)
			THEN 'Resource Database'
			ELSE DB_NAME ([database_id]) END) AS DatabaseName, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 1 ELSE 0 END) AS DirtyPageCount, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 0 ELSE 1 END) AS CleanPageCount
	FROM sys.dm_os_buffer_descriptors
	GROUP BY [database_id]) AS buffers
ORDER BY DatabaseName
GO 

-- Explain about tempdb checkpoints

-- What about getting a view of what's in
-- memory per *table*?
SELECT * FROM sys.dm_os_buffer_descriptors;
GO

-- All we get is an allocation unit ID

-- For a single database, with names
USE SalesDB;
GO
SELECT
	OBJECT_NAME (p.object_id) AS ObjectName,
	p.index_id,
	i.name,
	i.type_desc,
	au.type_desc,
	DirtyPageCount,
	CleanPageCount,
	DirtyPageCount * 8 / 1024 AS DirtyPageMB,
	CleanPageCount * 8 / 1024 AS CleanPageMB
FROM
	(SELECT
		allocation_unit_id,
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 1 ELSE 0 END) AS DirtyPageCount, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 0 ELSE 1 END) AS CleanPageCount
	FROM sys.dm_os_buffer_descriptors
	WHERE database_id = DB_ID ('SalesDB')
	GROUP BY allocation_unit_id) AS buffers
INNER JOIN sys.allocation_units AS au
	ON au.allocation_unit_id = buffers.allocation_unit_id
INNER JOIN sys.partitions AS p
	ON au.container_id = p.partition_id
INNER JOIN sys.indexes AS i
	ON i.index_id = p.index_id
		AND p.object_id = i.object_id
WHERE p.object_id > 100
ORDER BY ObjectName, p.index_id;
GO

-- How about aggregating the empty space?
SELECT * FROM sys.dm_os_buffer_descriptors;
GO

SELECT
	COUNT (*) * 8 / 1024 AS MBUsed,
	SUM (free_space_in_bytes) / (1024 * 1024) AS MBEmpty
FROM sys.dm_os_buffer_descriptors;
GO

-- And by database
SELECT 
	(CASE WHEN ([database_id] = 32767)
		THEN 'Resource Database'
		ELSE DB_NAME ([database_id]) END) AS DatabaseName,
	COUNT (*) * 8 / 1024 AS MBUsed,
	SUM (free_space_in_bytes) / (1024 * 1024) AS MBEmpty

FROM sys.dm_os_buffer_descriptors
GROUP BY [database_id];
GO

-- Dropping all pages for a single database
ALTER DATABASE SalesDB SET OFFLINE;
GO
ALTER DATABASE SalesDB SET ONLINE;
GO

SELECT *,
	DirtyPageCount * 8 / 1024 AS DirtyPageMB,
	CleanPageCount * 8 / 1024 AS CleanPageMB
FROM
	(SELECT 
		(CASE WHEN ([database_id] = 32767)
			THEN 'Resource Database'
			ELSE DB_NAME ([database_id]) END) AS DatabaseName, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 1 ELSE 0 END) AS DirtyPageCount, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 0 ELSE 1 END) AS CleanPageCount
	FROM sys.dm_os_buffer_descriptors
	GROUP BY [database_id]) AS buffers
ORDER BY DatabaseName
GO

-- How much data does CHECKDB read in?
DBCC CHECKDB (SalesDB);
GO

-- Looks like at least 31000 pages, right?

SELECT *,
	DirtyPageCount * 8 / 1024 AS DirtyPageMB,
	CleanPageCount * 8 / 1024 AS CleanPageMB
FROM
	(SELECT 
		(CASE WHEN ([database_id] = 32767)
			THEN 'Resource Database'
			ELSE DB_NAME ([database_id]) END) AS DatabaseName, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 1 ELSE 0 END) AS DirtyPageCount, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 0 ELSE 1 END) AS CleanPageCount
	FROM sys.dm_os_buffer_descriptors
	GROUP BY [database_id]) AS buffers
ORDER BY DatabaseName
GO

-- Wrong! Explain about disfavoring.

-- What about a backup?
BACKUP DATABASE SalesDB TO
	DISK = 'C:\SQLskills\TmpSalesDBBackup.bck'
	WITH INIT;
GO

SELECT *,
	DirtyPageCount * 8 / 1024 AS DirtyPageMB,
	CleanPageCount * 8 / 1024 AS CleanPageMB
FROM
	(SELECT 
		(CASE WHEN ([database_id] = 32767)
			THEN 'Resource Database'
			ELSE DB_NAME ([database_id]) END) AS DatabaseName, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 1 ELSE 0 END) AS DirtyPageCount, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 0 ELSE 1 END) AS CleanPageCount
	FROM sys.dm_os_buffer_descriptors
	GROUP BY [database_id]) AS buffers
ORDER BY DatabaseName
GO

-- Notice the dirty page count increased?

SELECT * FROM sys.dm_os_buffer_descriptors
WHERE [database_id] = DB_ID ('SalesDB')
	AND [is_modified] = 1;
GO

-- What's the data page?
DBCC TRACEON (3604);
DBCC PAGE (SalesDB, 1, XX, 3)

SELECT OBJECT_NAME (24);
GO

-- Doesn't use the buffer pool

-- What about when using WITH CHECKSUM
BACKUP DATABASE SalesDB TO
	DISK = 'C:\SQLskills\TmpSalesDBBackup.bck'
	WITH INIT, CHECKSUM;
GO

SELECT *,
	DirtyPageCount * 8 / 1024 AS DirtyPageMB,
	CleanPageCount * 8 / 1024 AS CleanPageMB
FROM
	(SELECT 
		(CASE WHEN ([database_id] = 32767)
			THEN 'Resource Database'
			ELSE DB_NAME ([database_id]) END) AS DatabaseName, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 1 ELSE 0 END) AS DirtyPageCount, 
		SUM (CASE WHEN ([is_modified] = 1)
			THEN 0 ELSE 1 END) AS CleanPageCount
	FROM sys.dm_os_buffer_descriptors
	GROUP BY [database_id]) AS buffers
ORDER BY DatabaseName
GO
